#include "serial.h"

#include <stdio.h>

static HANDLE hComPort;

//---------------------------------------------------------------------------
void openComPort(const int comPortNumber)
{
	wchar_t comPortName[255];
	COMMTIMEOUTS tout;
	DCB          dcb;

	// use the format "\\.\COM10" to allow COM ports greater than 9
	swprintf(comPortName, L"\\\\.\\COM%d", comPortNumber);

	hComPort = CreateFileW(comPortName, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);

	if (hComPort != INVALID_HANDLE_VALUE) {

		// set up communication time outs
		tout.ReadTotalTimeoutConstant    = 100;
		tout.ReadTotalTimeoutMultiplier  = 0;
		tout.WriteTotalTimeoutMultiplier = 0;
		tout.WriteTotalTimeoutConstant   = 0;
		SetCommTimeouts(hComPort, &tout);

		// set up Baudrate to 9600 8N1
		GetCommState(hComPort, &dcb);

		dcb.DCBlength = sizeof(DCB);
		dcb.BaudRate  = CBR_9600;      // 9600 bits/second
		dcb.ByteSize  = 8;             // 8 data bits
		dcb.fParity   = 0;             // no parity
		dcb.StopBits  = ONESTOPBIT;    // 1 stop bit

		SetCommState(hComPort, &dcb);
	}
}

//---------------------------------------------------------------------------
void closeComPort(void)
{
	CloseHandle(hComPort);
}

//---------------------------------------------------------------------------
int writeComPort(const void *Buffer, DWORD NumBytes)
{
	DWORD bytesWritten;
	BOOL  status;

	// Sanity check on the Handle
	if ( (hComPort == 0) || (hComPort == INVALID_HANDLE_VALUE) ) {
		return 0;
	}

	// Output the data to the Comport
	status = WriteFile(hComPort, Buffer, NumBytes, &bytesWritten, 0);

	return bytesWritten;
}

//---------------------------------------------------------------------------
int readComPort(DWORD bufferSize, void *buffer)
{
	COMSTAT comStatus;
	DWORD   comErrors;
	DWORD   bytesToRead;
	DWORD   bytesRead = 0;

	if ( (hComPort == 0) || (hComPort == INVALID_HANDLE_VALUE) ) {
		return -1;
	}

	// Check the input buffer
	ClearCommError(hComPort, &comErrors, &comStatus);
	bytesToRead = comStatus.cbInQue;

	// ReadFile() only if there is data.
	if (bytesToRead > 0) {

		// Make sure we do not overrun the Buffer
		if (bytesToRead > bufferSize) {
			bytesToRead = bufferSize;
		}

		// Read the data, no (NULL) Overlapped I/O
		if (!ReadFile(hComPort, buffer, bytesToRead, &bytesRead, NULL)) {
			return -1;
		}
	}

	return bytesRead;
}

//---------------------------------------------------------------------------
int writeCommand(const char *command)
{
	char cmd[256];
	char buf[10] = { 0 };
	int  timeout;
	int  rb;
	int  len;

	strncpy(cmd, command, 250);
	cmd[250] = 0;

	// terminate command with CR LF
	if (!strstr(cmd, "\r\n")) {
		strcat(cmd, "\r\n");
	}

	len = strlen(cmd);

	for (int i = 0; i < len; i++) {
		writeComPort(&cmd[i], 1);

		// read echo
		timeout = 10;
		do {
			rb = readComPort(1, buf);
			Sleep(10);
			timeout--;
		} while ( (rb == 0) && (timeout > 0) );

		if (cmd[i] != buf[0]) {
			return 0;
		}
	}

	Sleep(20);

	return 1;
}

//---------------------------------------------------------------------------
int writeCommandAndReadAnswer(const char *command, char *answer)
{
	int  bytesRead;
	char inbuf[256];
	int  to = 0;
	char *p;

	if (!answer) {
		return 0;
	}

	// write command to HV device
	if (!writeCommand(command)) {
		return 0;
	}

	// init buffer
	*answer = 0;

	// read answer (terminated with CR + LF)
	to = 0;
	do {
		bytesRead = readComPort(sizeof(inbuf), inbuf);

		if (bytesRead > 0) {
			inbuf[bytesRead] = 0;
			strcat(answer, inbuf);
			to = 0;
		}

		Sleep(10);
	} while ( (strstr(answer, "\r\n") == 0) && (++to < 80) ); // wait for CR+LF or timeout

	// clear trailing CR + LF
	if ( p  = strstr(answer, "\r\n") ) {
		*p = 0;
	}

	return 1;
}
